package gui;

import clientP2P.*;
import clientP2P.ClientInterface;
import clientP2P.ClientManagement;
import tools.SearchFile;
import tools.LogLevel;
import tools.Logger;
import localException.ProtocolError;
import localException.InternalError;
import localException.ProtocolError;
import localException.SizeError;
import localException.TransmissionError;
import localException.VersionError;
import localException.SocketClosed;
import remoteException.EmptyFile;
import remoteException.EmptyDirectory;
import remoteException.InternalRemoteError;
import remoteException.NotFound;
import remoteException.ProtocolRemoteError;
import remoteException.VersionRemoteError;
import remoteException.NotATracker;
import remoteException.UnknownHost;
import java.io.IOException;
import java.net.UnknownHostException;
import java.net.SocketException;
import javax.swing.JFrame;

/** Implementation of P2P-JAVA-PROJECT CLIENT interface for CLI
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class ClientInterfaceGUI extends ClientInterface {

	/** Constructor with clientManagement, logger and scanner.
	 * @param clientManagement	ClientManagement used
	 * @param logger	Logger used
	 */

	 Logger loggerC;

	public ClientInterfaceGUI(ClientManagement clientManagement, Logger logger) {
		super(clientManagement, logger);
		this.loggerC = logger;
	}

	/** Implementation of Runnable
	 */


	 public void run(){
		 boolean isRunning = initHostList();
		 if (isRunning) {
			 try{
				 int i = 1;
				 SearchFile searchEngine = new SearchFile();
				 int optionSearch = 0;
				 String searchInput = "";
				 String[] list = clientManagement.listDirectory();
				 //System.out.println("running ClientInterfaceGUI and creating window");
				 //for(String listItem: list){
				 //System.out.println(listItem);
				 //}
				 JFrame win = new JFrame();
				 win.setTitle("Client");
				 win.setSize(550, 250);
				 win.setLocationRelativeTo(null);
				 win.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				 win.add(new DownloadSelectionGen(list, clientManagement, loggerC));
				 win.setVisible(true);
			 } catch (EmptyDirectory e) {
					writeLog("Server has no file in directory", LogLevel.Error);
					ErrorFrame err = new ErrorFrame("Server has no file in directory");
				}
		 catch (InternalError e) {
				writeLog("Client internal error", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Client internal error");
			} catch (UnknownHostException e) {
				writeLog("Server host is unknown", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Server host is unknown");
			} catch (IOException e) {
				writeLog("Request cannot be send or response cannot be received", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Request cannot be send or response cannot be received");
			} catch (TransmissionError e) {
				writeLog("Message received is too big", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Message received is too big");
			} catch (ProtocolError e) {
				writeLog("Cannot decode server’s response", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Cannot decode server’s response");
			} catch (VersionError e) {
				writeLog("Server’s response use bad version of the protocol", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Server’s response use bad version of the protocol");
			} catch (SizeError e) {
				writeLog("Cannot handle this packets because of internal representation limitations of numbers on the client", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Cannot handle this packets because of internal representation limitations of numbers on the client");
			} catch (InternalRemoteError e) {
				writeLog("Server internal error", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Server internal error");
			} catch (ProtocolRemoteError e) {
				writeLog("Server cannot decode client’s request", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Server cannot decode client’s request");
			} catch (VersionRemoteError e) {
				writeLog("Server cannot decode this version of the protocol", LogLevel.Error);
				ErrorFrame err = new ErrorFrame("Server cannot decode this version of the protocol");
			}
	 }
 }


	/** Initialization of hostList with retry in failure
	 * @return	true if successfully initialized
	 */
	private boolean initHostList() {
		boolean contacted = false;
		boolean firstLoop = true;
		boolean stop = false;
		while (!contacted && !stop) {
			try {
				if (!firstLoop) {
					writeLog("Cannot contact tracker... ", LogLevel.Error);
				}
				firstLoop = false;
				clientManagement.initHostList();
				contacted = true;
			} catch (SocketException e) {
			} catch (UnknownHostException e) {
			} catch (IOException e) {
			} catch (Exception e) {
				return false;
			}
		}
		return !stop;
	}


}
